@@ -1,16 +1,24 @@
 #include <unistd.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <sys/stat.h>
 #include <pwd.h>
 #include <dlfcn.h>
 #include <mach-o/dyld.h>
 #include <CoreFoundation/CoreFoundation.h>
 #include <ApplicationServices/ApplicationServices.h>
+#include <locale.h>
 
 /*
     Typedefs
 */
 
 typedef int PyObject;
+typedef PyObject* (*Py_BuildValuePtr)(const char*, ...);
+#if 0
+typedef void (*Py_SetPathPtr)(const wchar_t*);
+#endif
+typedef int (*PySys_SetObjectPtr)(const char*, PyObject*);
 typedef void (*Py_SetProgramNamePtr)(const char *);
 typedef void (*Py_InitializePtr)(void); 
 typedef int (*PyRun_SimpleFilePtr)(FILE *, const char *);
@@ -511,10 +519,10 @@ static void setResourcePath(void) {
 }
 
 static void setExecutablePath(void) {
-    char executable_path[PATH_MAX];
+    char executable_path[PATH_MAX+1];
     uint32_t bufsize = PATH_MAX;
-    if (!_NSGetExecutablePath(executable_path, &bufsize)) {
-        executable_path[bufsize] = '\0';
+    memset(executable_path, '\0', PATH_MAX+1);
+    if (_NSGetExecutablePath(executable_path, &bufsize) == 0) {
         setenv("EXECUTABLEPATH", executable_path, 1);
     }
 }
@@ -838,6 +846,8 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     void *py_dylib;
     int rval;
     FILE *mainScriptFile;
+    char* curenv = NULL;
+    char* curlocale = NULL;
 
 
     if (!py2app_getApplicationName()) return report_error(ERR_NONAME);
@@ -880,6 +890,9 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
 	    if (!py2app_ ## NAME) { \
 		return report_linkEdit_error(); \
 	    } 
+
+#define OPT_LOOKUP(NAME) \
+	    NAME ## Ptr py2app_ ## NAME = (NAME ## Ptr)dlsym(py_dylib, #NAME); 
     
     LOOKUP(Py_SetProgramName);
     LOOKUP(Py_Initialize);
@@ -888,13 +901,129 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     LOOKUP(PySys_GetObject);
     LOOKUP(PySys_SetArgv);
     LOOKUP(PyObject_GetAttrString);
+    LOOKUP(Py_BuildValue);
+#if 0
+    OPT_LOOKUP(Py_SetPath);
+#endif
+    LOOKUP(PySys_SetObject);
 
+
+    int isPy3K = dlsym(py_dylib, "PyBytes_FromString") != NULL;
+
+#undef OPT_LOOKUP
 #undef LOOKUP
+	
+    /*
+     * When apps are started from the Finder (or anywhere
+     * except from the terminal), the LANG and LC_* variables
+     * aren't set in the environment. This confuses Py_Initialize
+     * when it tries to import the codec for UTF-8, 
+     * therefore explicitly set the locale. 
+     *
+     * Also set the LC_CTYPE environment variable because Py_Initialize
+     * reset the locale information using the environment :-(
+     */
+    curlocale = setlocale(LC_ALL, NULL);
+    if (curlocale != NULL) {
+      curlocale = strdup(curlocale);
+      if (curlocale == NULL) {
+        (void)report_error("cannot save locale information");
+	return -1;
+      }
+    }
+    setlocale(LC_ALL, "en_US.UTF-8");
 
-    py2app_Py_SetProgramName(c_pythonInterpreter);
+    curenv = getenv("LC_CTYPE");
+    if (curenv) {
+	    curenv = strdup(curenv);
+    }
+    setenv("LC_CTYPE", "en_US.UTF-8", 1);
+
+    if (isPy3K) {
+#if 0
+	/*
+	 *  Py_SetPath is disabled for now, py2app works without
+	 *  it and the code below requires changes to work properly
+	 *  in all cases (crashes with alias and semi-standalone,
+	 *  and a hardcoded version string in the path string).
+	 */
+    	if (py2app_Py_SetPath != NULL) {
+		size_t len = (strlen(resource_path) * 3) + 256;
+		char* search_path = malloc(len);
+		wchar_t* w_search_path;
+
+		if (search_path == NULL) {
+			report_error("cannot allocate sys.path");
+			return -1;
+		}
+		/* FIXME: should use actual python version */
+		snprintf(search_path, len,
+			"%s/lib/python3.2:%s/lib/python32.zip:%s/lib/python3.2/lib-dynload",
+			resource_path, resource_path, resource_path);
+
+		w_search_path = (wchar_t*)malloc(len * sizeof(wchar_t));
+		if (search_path == NULL) {
+			report_error("cannot allocate wide sys.path");
+			return -1;
+		}
+
+		mbstowcs(w_search_path, search_path, len);
+		free(search_path); search_path = NULL;
+
+		py2app_Py_SetPath(w_search_path);
+		free(w_search_path);
+
+		/* sys.prefix and sys.exec_prefix aren't set yet */
+		
+	}
+#endif
+
+    	wchar_t w_pythonInterpreter[PATH_MAX+1];
+    	mbstowcs(w_pythonInterpreter, c_pythonInterpreter, PATH_MAX+1);
+    	py2app_Py_SetProgramName((char*)w_pythonInterpreter);
+
+
+
+    } else {
+	    py2app_Py_SetProgramName(c_pythonInterpreter);
+    }
 
     py2app_Py_Initialize();
 
+#if 0
+    if (isPy3K && py2app_Py_SetPath) {
+	    PyObject* prefix = py2app_Py_BuildValue(
+		"s", resource_path);
+	    if (prefix == NULL) {
+        	rval = report_script_error(ERR_PYTHONEXCEPTION);
+		return rval;
+	    }
+
+	    if (py2app_PySys_SetObject("prefix", prefix) == -1) {
+        	rval = report_script_error(ERR_PYTHONEXCEPTION);
+		return rval;
+	    }
+	    if (py2app_PySys_SetObject("exec_prefix", prefix) == -1) {
+        	rval = report_script_error(ERR_PYTHONEXCEPTION);
+		return rval;
+	    }
+     }
+#endif
+
+
+    /*
+     * Reset the environment and locale information
+     */
+    setlocale(LC_CTYPE, curlocale);
+    free(curlocale);
+
+    if (curenv) {
+	    setenv("LC_CTYPE", curenv, 1);
+	    free(curenv);
+    } else {
+	    unsetenv("LC_CTYPE");
+    }
+
     py2app_CFStringGetCString(
         mainScript, c_mainScript,
         sizeof(c_mainScript), kCFStringEncodingUTF8);
@@ -906,6 +1035,7 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     memcpy(&argv_new[1], &argv[1], (argc - 1) * sizeof(char *));
     py2app_PySys_SetArgv(argc, argv_new);
 
+
     mainScriptFile = fopen(c_mainScript, "r");
     rval = py2app_PyRun_SimpleFile(mainScriptFile, c_mainScript);
     fclose(mainScriptFile);
