@@ -13,6 +13,11 @@ import zipfile
 import plistlib
 import shlex
 import shutil
+
+try:
+    import sysconfig
+except ImportError:
+    sysconfig = None
 from cStringIO import StringIO
 
 from itertools import chain, imap
@@ -26,7 +31,8 @@ from distutils.errors import *
 from altgraph.compat import *
 
 from modulegraph.find_modules import find_modules, parse_mf_results
-from modulegraph.modulegraph import SourceModule, Package, os_listdir
+from modulegraph.modulegraph import SourceModule, Package
+from modulegraph import zipio
 
 import macholib.dyld
 import macholib.MachOStandalone
@@ -37,7 +43,7 @@ from py2app.util import \
     fancy_split, byte_compile, make_loader, imp_find_module, \
     copy_tree, fsencoding, strip_files, in_system_path, makedirs, \
     iter_platform_files, find_version, skipscm, momc, copy_file, \
-    os_path_isdir, copy_resource
+    copy_resource
 from py2app.filters import \
     not_stdlib_filter, not_system_filter, has_filename_filter
 from py2app import recipes
@@ -208,7 +214,7 @@ class py2app(Command):
         ("alias", 'A',
          "Use an alias to current source file (for development only!)"),
         ("argv-emulation", 'a',
-         "Use argv emulation [disabled for plugins]"),
+         "Use argv emulation [disabled for plugins]. Does not work with python 3.x"),
         ("argv-inject=", None,
          "Inject some commands into the argv"),
         ("use-pythonpath", None,
@@ -284,6 +290,12 @@ class py2app(Command):
         self.eggs = []
 
     def finalize_options (self):
+        if self.argv_emulation:
+            if sys.version_info[0] == 3:
+                raise DistutilsOptionError("argv-emulation is not supported on python 3.x")
+            elif sys.maxint > 2 ** 32:
+                raise DistutilsOptionError("argv-emulation is not supported for 64-bit executables")
+
         if not self.strip:
             self.no_strip = True
         elif self.no_strip:
@@ -293,6 +305,9 @@ class py2app(Command):
             self.argv_inject = shlex.split(self.argv_inject)
         self.includes = set(fancy_split(self.includes))
         self.includes.add('encodings.*')
+        #if sys.version_info[:2] >= (3, 2):
+        #    self.includes.add('pkgutil')
+        #    self.includes.add('imp')
         self.packages = set(fancy_split(self.packages))
         self.excludes = set(fancy_split(self.excludes))
         self.excludes.add('readline')
@@ -421,6 +436,12 @@ class py2app(Command):
             yield runtime
 
     def run(self):
+        if hasattr(self.distribution, "install_requires") \
+                and self.distribution.install_requires:
+
+            self.distribution.fetch_build_eggs(self.distribution.install_requires)
+
+
         build = self.reinitialize_command('build')
         build.build_base = self.bdist_base
         build.run()
@@ -603,6 +624,7 @@ class py2app(Command):
             else:
                 self.run_normal()
         except:
+            raise
             # XXX - remove when not debugging
             #       distutils sucks
             import pdb, sys, traceback
@@ -641,6 +663,18 @@ class py2app(Command):
             mf.graphreport(open(dgraph, 'w'), flatpackages=flatpackages)
 
     def finalize_modulefinder(self, mf):
+        for item in mf.flatten():
+            if isinstance(item, Package) and item.filename == '-':
+                fn = os.path.join(self.temp_dir, 'empty_package', '__init__.py')
+                if not os.path.exists(fn):
+                    dn = os.path.dirname(fn)
+                    if not os.path.exists(dn):
+                        os.makedirs(dn)
+                    fp = open(fn, 'w')
+                    fp.close()
+
+                item.filename = fn
+
         py_files, extensions = parse_mf_results(mf)
         py_files = list(py_files)
         extensions = list(extensions)
@@ -725,7 +759,8 @@ class py2app(Command):
                 fn = os.path.realpath(getattr(ext, 'filename', None))
                 pkgexts.append(ext)
             else:
-                py_files.append(self.create_loader(ext))
+                if '.' in ext.identifier:
+                    py_files.append(self.create_loader(ext))
                 copyexts.append(ext)
             extmap[fn] = ext
 
@@ -829,16 +864,19 @@ class py2app(Command):
 
         target_dir = os.path.join(target_dir, *(package.identifier.split('.')))
         for dname in package.packagepath:
-            filenames = filter(datafilter, os_listdir(dname))
+            filenames = filter(datafilter, zipio.listdir(dname))
             for fname in filenames:
                 if fname in ('.svn', 'CVS'):
                     # Scrub revision manager junk
                     continue
+                if fname in ('__pycache__',):
+                    # Ignore PEP 3147 bytecode cache
+                    continue
                 pth = os.path.join(dname, fname)
 
                 # Check if we have found a package, exclude those
-                if os_path_isdir(pth):
-                    for p in os_listdir(pth):
+                if zipio.isdir(pth):
+                    for p in zipio.listdir(pth):
                         if p.startswith('__init__.') and p[8:] in exts:
                             break
 
@@ -935,6 +973,24 @@ class py2app(Command):
         # XXX - In this particular case we know exactly what we can
         #       get away with.. should this be extended to the general
         #       case?  Per-framework recipes?
+        includedir = None
+        configdir = None
+        if sysconfig is not None:
+            includedir = sysconfig.get_config_var('CONFINCLUDEPY')
+            configdir = sysconfig.get_config_var('LIBPL')
+
+
+        if includedir is None:
+            includedir = 'python%d.%d'%(sys.version_info[:2])
+        else:
+            includedir = os.path.basename(includedir)
+
+        if configdir is None:
+            configdir = 'config'
+        else:
+            configdir = os.path.basename(configdir)
+
+
         indir = os.path.dirname(os.path.join(info['location'], info['name']))
         outdir = os.path.dirname(os.path.join(dst, info['name']))
         self.mkpath(os.path.join(outdir, 'Resources'))
@@ -943,15 +999,17 @@ class py2app(Command):
         # distutils looks for some files relative to sys.executable, which
         # means they have to be in the framework...
         self.mkpath(os.path.join(outdir, 'include'))
-        self.mkpath(os.path.join(outdir, 'include', pydir))
+        self.mkpath(os.path.join(outdir, 'include', includedir))
         self.mkpath(os.path.join(outdir, 'lib'))
         self.mkpath(os.path.join(outdir, 'lib', pydir))
-        self.mkpath(os.path.join(outdir, 'lib', pydir, 'config'))
+        self.mkpath(os.path.join(outdir, 'lib', pydir, configdir))
+
+
         fmwkfiles = [
             os.path.basename(info['name']),
             'Resources/Info.plist',
-            'include/%s/pyconfig.h'%(pydir,),
-            'lib/%s/config/Makefile'%(pydir,)
+            'include/%s/pyconfig.h'%(includedir),
+            'lib/%s/%s/Makefile'%(pydir, configdir)
         ]
         for fn in fmwkfiles:
             self.copy_file(
@@ -1020,6 +1078,11 @@ class py2app(Command):
 
     def initialize_prescripts(self):
         prescripts = []
+        if 0 and sys.version_info[:2] >= (3, 2) and not self.alias:
+            # Python 3.2 or later requires a more complicated
+            # bootstrap
+            prescripts.append('import_encodings')
+
         if self.site_packages or self.alias:
             prescripts.append('site_packages')
 
@@ -1183,18 +1246,13 @@ class py2app(Command):
         self.compile_datamodels(resdir)
         self.compile_mappingmodels(resdir)
 
-        from Carbon.File import FSRef
-        aliasdata = FSRef(script).FSNewAliasMinimal().data
-
         bootfn = '__boot__'
         bootfile = open(os.path.join(resdir, bootfn + '.py'), 'w')
         for fn in target.prescripts:
             bootfile.write(self.get_bootstrap_data(fn))
             bootfile.write('\n\n')
         bootfile.write('try:\n')
-        bootfile.write('    _run((%r, %r))\n' % (
-            aliasdata, os.path.realpath(script),
-        ))
+        bootfile.write('    _run(%r)\n' % os.path.realpath(script))
         bootfile.write('except KeyboardInterrupt:\n')
         bootfile.write('    pass\n')
         bootfile.close()
@@ -1209,10 +1267,22 @@ class py2app(Command):
         self.resdir = resdir
         self.plist = plist
 
-        for src, dest in self.iter_data_files():
-            dest = os.path.join(resdir, dest)
-            self.mkpath(os.path.dirname(dest))
-            copy_resource(src, dest, dry_run=self.dry_run)
+        includedir = None
+        configdir = None
+        if sysconfig is not None:
+            includedir = sysconfig.get_config_var('CONFINCLUDEPY')
+            configdir = sysconfig.get_config_var('LIBPL')
+
+
+        if includedir is None:
+            includedir = 'python%d.%d'%(sys.version_info[:2])
+        else:
+            includedir = os.path.basename(includedir)
+
+        if configdir is None:
+            configdir = 'config'
+        else:
+            configdir = os.path.basename(configdir)
 
         self.compile_datamodels(resdir)
         self.compile_mappingmodels(resdir)
@@ -1234,8 +1304,8 @@ class py2app(Command):
         realhome = os.path.join(sys.prefix, 'lib', 'python' + sys.version[:3])
         self.mkpath(pydir)
         self.symlink('../../site.py', os.path.join(pydir, 'site.py'))
-        cfgdir = os.path.join(pydir, 'config')
-        realcfg = os.path.join(realhome, 'config')
+        cfgdir = os.path.join(pydir, configdir)
+        realcfg = os.path.join(realhome, configdir)
         real_include = os.path.join(sys.prefix, 'include')
         if self.semi_standalone:
             self.symlink(realcfg, cfgdir)
@@ -1247,10 +1317,10 @@ class py2app(Command):
                 if os.path.exists(rfn):
                     self.copy_file(rfn, os.path.join(cfgdir, fn))
 
-            inc_dir = os.path.join(resdir, 'include', 'python' + sys.version[:3])
+            inc_dir = os.path.join(resdir, 'include', includedir)
             self.mkpath(inc_dir)
-            self.copy_file(os.path.join(real_include, 'python%s/pyconfig.h'%(
-                sys.version[:3])), os.path.join(inc_dir, 'pyconfig.h'))
+            self.copy_file(os.path.join(real_include, '%s/pyconfig.h'%(
+                includedir)), os.path.join(inc_dir, 'pyconfig.h'))
 
 
         self.copy_file(arcname, arcdir)
@@ -1276,6 +1346,35 @@ class py2app(Command):
             self.mkpath(os.path.dirname(fn))
             copy_file(copyext.filename, fn, dry_run=self.dry_run)
 
+        if 0 and sys.version_info[:2] >= (3, 2) and not self.alias:
+            import encodings
+            import encodings.cp437
+            import encodings.utf_8
+            import encodings.latin_1
+            import codecs
+
+            encodings_dir = os.path.join(pydir, 'encodings')
+            self.mkpath(encodings_dir)
+
+            byte_compile([
+                    SourceModule('encodings.__init__', encodings.__file__),
+                    SourceModule('encodings.cp437', encodings.cp437.__file__),
+                    SourceModule('encodings.utf_8', encodings.utf_8.__file__),
+                    SourceModule('encodings.latin_1', encodings.latin_1.__file__),
+                    SourceModule('codecs', codecs.__file__),
+                ],
+                target_dir=pydir,
+                optimize=self.optimize,
+                force=self.force,
+                verbose=self.verbose,
+                dry_run=self.dry_run)
+
+            if not self.dry_run:
+                fp = open(os.path.join(encodings_dir, 'aliases.py'), 'w')
+                fp.write('aliases = {}\n')
+                fp.close()
+
+
         target.appdir = appdir
         return appdir
 
