@@ -8,7 +8,21 @@ gPreBuildVariants = [
     {
         'name': 'main-universal',
         'target': '10.5',
-        'cflags': '-isysroot /Developer/SDKs/MacOSX10.5.sdk -arch i386 -arch ppc -arch ppc64 -arch x86_64',
+        #'cflags': '-isysroot /Developer/SDKs/MacOSX10.5.sdk -arch i386 -arch ppc -arch ppc64 -arch x86_64',
+        'cflags': '-isysroot /Xcode4/SDKs/MacOSX10.5.sdk -arch i386 -arch ppc -arch ppc64 -arch x86_64',
+        'cc': 'gcc-4.2',
+    },
+    {
+        'name': 'main-ppc64',
+        'target': '10.5',
+        #'cflags': '-isysroot /Developer/SDKs/MacOSX10.5.sdk -arch x86_64',
+        'cflags': '-isysroot /Xcode4/SDKs/MacOSX10.5.sdk -arch x86_64',
+        'cc': 'gcc-4.2',
+    },
+    {
+        'name': 'main-x86_64',
+        'target': '10.5',
+        'cflags': '-isysroot / -arch x86_64',
         'cc': 'gcc-4.2',
     },
     {
@@ -24,15 +38,33 @@ gPreBuildVariants = [
         'cc': 'gcc-4.2',
     },
     {
-        'name': 'main-fat',
-        'target': '10.3',
-        'cflags': '-isysroot /Developer/SDKs/MacOSX10.4u.sdk -arch i386 -arch ppc',
+        'name': 'main-i386',
+        #'target': '10.3',
+        #'cflags': '-isysroot /Developer/SDKs/MacOSX10.4u.sdk -arch i386',
+        'target': '10.4',
+        'cflags': '-isysroot /Developer/SDKs/MacOSX10.6.sdk -arch i386',
         'cc': 'gcc-4.0',
     },
+    {
+        'name': 'main-ppc',
+        #'target': '10.3',
+        #'cflags': '-isysroot /Developer/SDKs/MacOSX10.4u.sdk -arch ppc',
+        'target': '10.4',
+        'cflags': '-isysroot /Xcode4/SDKs/MacOSX10.6.sdk -arch ppc',
+        'cc': 'gcc-4.2',
+    },
+    {
+        'name': 'main-fat',
+        #'target': '10.3',
+        #'cflags': '-isysroot /Developer/SDKs/MacOSX10.4u.sdk -arch i386 -arch ppc',
+        'target': '10.4',
+        'cflags': '-isysroot /Xcode4/SDKs/MacOSX10.6.sdk -arch i386 -arch ppc',
+        'cc': 'gcc-4.2',
+    },
 ]
 
 
-def main():
+def main(all=False):
     basepath = os.path.dirname(__file__)
     builddir = os.path.join(basepath, 'prebuilt')
     if not os.path.exists(builddir):
@@ -43,6 +75,7 @@ def main():
 
     BASE_CFLAGS = cfg['CFLAGS']
     BASE_CFLAGS = BASE_CFLAGS.replace('-dynamic', '')
+    BASE_CFLAGS += ' -bundle -framework Foundation -framework AppKit'
     while True:
         x = re.sub('-arch\s+\S+', '', BASE_CFLAGS)
         if x == BASE_CFLAGS:
@@ -55,29 +88,31 @@ def main():
             break
         BASE_CFLAGS=x
 
-    BASE_CFLAGS += ' -bundle -framework Foundation -framework AppKit'
+    arch = distutils.util.get_platform().split('-')[-1]
+    if sys.prefix.startswith('/System') and \
+            sys.version_info[:2] == (2,5):
+        arch = "fat"
+
+    name = 'main-' + arch
 
     for entry in gPreBuildVariants:
+        if (not all) and entry['name'] != name: continue
+
         CC=entry['cc']
         CFLAGS = BASE_CFLAGS + ' ' + entry['cflags']
         os.environ['MACOSX_DEPLOYMENT_TARGET'] = entry['target']
         dest = os.path.join(builddir, entry['name'])
         if not os.path.exists(dest) or (
                 os.stat(dest).st_mtime < os.stat(src).st_mtime):
-            os.system('"%(CC)s" -arch i386 -arch ppc -o "%(dest)s" "%(src)s" %(CFLAGS)s' % locals())
-
-    arch = distutils.util.get_platform().split('-')[-1]
-
-    if sys.prefix.startswith('/System') and \
-            sys.version_info[:2] == (2,5):
-        arch = "fat"
+            os.system('"%(CC)s" -o "%(dest)s" "%(src)s" %(CFLAGS)s' % locals())
 
     dest = os.path.join(
-        builddir,
-        'main-' + arch
+            builddir,
+            'main-' + arch
     )
 
     return dest
 
+
 if __name__ == '__main__':
-    main()
+    main(all=True)
